/*
 * CAN.c
 *
 *  Created on: 23 Jul 2019
 *      Author: kentm
 */

// ******************** includes **********************************************
#include <stdint.h>
#include <string.h>
#include "CAN.h"

// ******************** typedefs and defines **********************************

// ******************** local variables ***************************************
static RCC_ClocksTypeDef g_RCC_Clocks;
// ******************** local function prototypes *****************************

// ******************** Implementation - local helper functions ***************

// ******************** Implementation - exported functions - API *************
void CANDeInit(CAN_Init_t * Init)
{
	GPIO_DeInit(Init->GPIO_PORT);
	RCC_AHB1PeriphClockCmd(Init->GPIO_AHB1Periph, DISABLE);
  RCC_APB1PeriphClockCmd(Init->CAN_APB1Periph, DISABLE);
}

void CANInit(CAN_Init_t * Init)
{
	RCC_AHB1PeriphClockCmd(Init->GPIO_AHB1Periph, ENABLE);
  //if(RCC_APB1Periph_CAN2 == Init->CAN_APB1Periph)
  	//  {RCC_APB1PeriphClockCmd(RCC_APB1Periph_CAN1	, ENABLE);}		// can 2 runs of can 1
  RCC_APB1PeriphClockCmd(Init->CAN_APB1Periph, ENABLE);
  GPIO_InitTypeDef GPIO_InitStructure;
  GPIO_PinAFConfig(Init->GPIO_PORT, Init->RX_GPIO_PinSource, Init->CAN_AF);
  GPIO_PinAFConfig(Init->GPIO_PORT, Init->TX_GPIO_PinSource, Init->CAN_AF);

  GPIO_InitStructure.GPIO_Pin = Init->RX_GPIO_Pin| Init->TX_GPIO_Pin;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_UP;
  GPIO_Init(Init->GPIO_PORT, &GPIO_InitStructure);

}

void CANswInit(PortCAN_t Port,CanardInstance* out_ins,void* mem_arena,size_t mem_arena_size,CanardOnTransferReception on_reception,
		CanardShouldAcceptTransfer should_accept,uint8_t self_node_id,const CanardSTM32AcceptanceFilterConfiguration* const filter_configs, uint16_t filtLen)
{
	CanardSTM32CANTimings timings;
	RCC_GetClocksFreq(&g_RCC_Clocks); //To make sure RCC is initialised properly
	int result = canardSTM32ComputeCANTimings(g_RCC_Clocks.PCLK1_Frequency, 1000000, &timings);
	if (result)
	{
		char *str = "compute timing failed\n";
		SerialTxQWait(str,strlen(str));
		//__ASM volatile("BKPT #01");
	}
	result = canardSTM32Init(&timings, CanardSTM32IfaceModeNormal, Port);
	if (result)
	{
		char *str = "init failed\n";
		SerialTxQWait(str,strlen(str));
		//__ASM volatile("BKPT #01");
	}
	if(filtLen)
	{
		canardSTM32ConfigureAcceptanceFilters(filter_configs,filtLen,Port);
		//char *str = "filter init\n";
		//SerialTxQWait(str,strlen(str));
	}

	canardInit(out_ins,                         // Uninitialized library instance
			mem_arena, // Raw memory chunk used for dynamic allocation
			mem_arena_size,      // Size of the above, in bytes
			on_reception, // Callback, see CanardOnTransferReception
			should_accept,          // Callback, see CanardShouldAcceptTransfer
			NULL);
	if(self_node_id)
	{canardSetLocalNodeID(out_ins,self_node_id);}
}

// ******************** end of CAN.c *********************************** 	
