/*
 * Que.c
 *
 *  Created on: 15 Aug 2019
 *      Author: kentm
 */

// ******************** includes **********************************************
#include <string.h>
#include "Que.h"
// ******************** typedefs and defines **********************************

// ******************** local variables ***************************************

// ******************** local function prototypes *****************************
// ******************** Implementation - local helper functions ***************
static inline uint16_t QNextInsert(Q_t *Q)
{
	return(((Q->insert+1)==Q->Len)?(0):(Q->insert+1));
}
static inline bool QFull(Q_t *Q)
{
	return(QNextInsert(Q) == Q->remove);
}
static inline bool QEmpty(Q_t *Q)
{
	return(Q->insert == Q->remove);
}
// ******************** Implementation - exported functions - API *************
void EnQ1(Q_t *Q, uint8_t val)
{
	if(QFull(Q))return;
	Q->Buffer[Q->insert] = val;
	Q->insert = QNextInsert(Q);
}

bool EnQ (Q_t *Q,uint8_t *data,uint16_t len)	// queue data block of len bytes, write how may written if not all
{
	uint16_t n1;
	if(QFree(Q)<len){return(false);}											// won't fit , sorry
	if(0 == len){return(false);}
	n1 = len;
	if(n1 > (Q->Len - Q->insert)){n1 = Q->Len-Q->insert;}								// copy to end of buffer if going to wrap
	memcpy(&Q->Buffer[Q->insert],data, n1);
	data += n1;																													// inc src pointer
	(len) -= n1;																												// dec len
	Q->insert += n1;																										// inc insert
	if (Q->insert >= Q->Len){Q->insert -= Q->Len;}											// wrap insert
	if (len != 0)																											// add any leftover bytes to the start of the ring buffer
	{
		memcpy(Q->Buffer, data, len);																		// copy last  section
		Q->insert = len;
	}
	return(true);
}
////////////////////////////////////////////////////////////////////////////////
uint16_t QLen(Q_t *Q)								// get the length of data in q (without dequeuing)
{
	uint16_t len = QFree(Q);
	return((Q->Len-1) - len);
}
////////////////////////////////////////////////////////////////////////////////
uint16_t QFree(Q_t *Q)								// get the length free space in q
{
	static uint16_t insert,remove;			// use temp vars in case of interrupt usage
	insert = Q->insert;
	remove = Q->remove;
	return((insert >= remove)?(Q->Len+remove-insert-1):(remove - insert-1));
}
////////////////////////////////////////////////////////////////////////////////
bool QMT(Q_t *Q)
{
	return(QEmpty(Q));
}
////////////////////////////////////////////////////////////////////////////////
bool DeQ1(Q_t *Q,uint8_t *byte)	// pull the next available byte from the queue
{
	if(!QEmpty(Q))
	{
		*byte = Q->Buffer[Q->remove];
		Q->remove ++;																			// inc remove point
		if (Q->remove >= Q->Len){Q->remove -= Q->Len;}		// wrap remove point
		return(true);
	}
	return(false);
}
bool DeQ (Q_t *Q,uint8_t *data,uint16_t *len)	// pull the next available data from the queue
{
	uint16_t n1;
	n1 = QLen(Q);
	if(n1 > *len){n1=*len;}
	*len = n1;
	if(0 == *len){return(false);}											// sorry, length is 0
	n1 = Q->Len - Q->remove;													// see how much we can copy from the tail of the buffer
	if (n1 >= *len)																		// if more than required
	{

		if(data)memcpy(data,&Q->Buffer[Q->remove], *len);				// copy all at once
	}
	else																							// else need to wrap
	{
		if(data)memcpy(data,&Q->Buffer[Q->remove], n1);					// copy end
		if(data)memcpy(data + n1,Q->Buffer, (*len) - n1);				// copy start
	}
	Q->remove += *len;																// inc remove point
	if (Q->remove >= Q->Len){Q->remove -= Q->Len;}		// wrap remove point
	return(true);
}
// ******************** end of Que.c *********************************** 	
